#include <Wire.h>
#include <Adafruit_PWMServoDriver.h>

Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver();

int servo1Pin = 0; // Conecta el servo1 al canal 0 del módulo PCA9685
int servo2Pin = 1; // Conecta el servo2 al canal 1 del módulo PCA9685
int servo3Pin = 2; // Conecta el servo3 al canal 2 del módulo PCA9685

const int triggerPin = 9;  // Pin digital al que está conectado el pin TRIGGER del sensor ultrasonido
const int echoPin = 10;    // Pin digital al que está conectado el pin ECHO del sensor ultrasonido

int umbral1 = 300;        // Umbral de iluminación para el primer sensor LDR
int umbral2 = 350;        // Umbral de iluminación para el segundo sensor LDR
int distanceThreshold = 20;  // Umbral de distancia en cm para detener los motores

void setup() {
  Wire.begin();
  pwm.begin();
  pwm.setPWMFreq(50);  // Establece la frecuencia del módulo PCA9685 a 50Hz

  pinMode(triggerPin, OUTPUT);  // Configura el pin TRIGGER del sensor ultrasonidos como salida
  pinMode(echoPin, INPUT);      // Configura el pin ECHO del sensor ultrasonidos como entrada
  
  pinMode(12, OUTPUT);
  pinMode(13, OUTPUT);
  pinMode(11, OUTPUT);
  
  maxAll(12, 13, 11, 11, 7);
  maxAll(12, 13, 11, 9, 0);
  maxAll(12, 13, 11, 12, 1);
  maxAll(12, 13, 11, 15, 0);
  int i = 0;
  for (i = 1; i <= 8; i++)
    maxAll(12, 13, 11, i, 0);
  maxAll(12, 13, 11, 10, 15);

  Serial.begin(9600);    // Inicia la comunicación serial a 9600 bps
}

void loop() {
  // Lee los valores de los sensores de luz
  int valorSensor1 = analogRead(A0);
  int valorSensor2 = analogRead(A1);
  Serial.print("Valor del sensor Derecha: ");
  Serial.println(valorSensor1);
  Serial.print("Valor del sensor Izquierda: ");
  Serial.println(valorSensor2);

  // Realiza la medición de la distancia con el sensor ultrasónico
    // Envía un pulso de ultrasonido
  digitalWrite(triggerPin, LOW);
  delayMicroseconds(2);
  digitalWrite(triggerPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(triggerPin, LOW);


  // Mide el tiempo de eco del pulso
  long duration = pulseIn(echoPin, HIGH);

  // Calcula la distancia en centímetros
  float distance = duration * 0.034 / 2;

  // Muestra la distancia en el monitor serie
  Serial.print("Distancia: ");
  Serial.print(distance);
  Serial.println(" cm");

  // Verifica si la distancia medida supera el umbral para detener los motores
  if (distance < distanceThreshold) {
    // Cara ataque
    expression(12, 13, 11, 153, 60, 90, 255, 255, 90, 60, 153);

    // Gira el servo3 a 180 grados y gira el robot
    setServoPosition(servo3Pin, 180);
    setServoPosition(servo1Pin, 180);
    setServoPosition(servo2Pin, 180);
  } else {
    setServoPosition(servo3Pin, 90);
    if ((valorSensor2 <= umbral2) && (valorSensor1 > umbral1)) {
      // Gira a la derecha
        setServoPosition(servo1Pin, 0);
        setServoPosition(servo2Pin, 90);
    } else  if ((valorSensor2 > umbral2) && (valorSensor1 <= umbral1)) {
        // Gira a la izquierda
       setServoPosition(servo1Pin, 90);
       setServoPosition(servo2Pin, 180);
    
    } else if ((valorSensor2 <= umbral2) && (valorSensor1 <= umbral1)) {
        // Todo recto
        setServoPosition(servo1Pin, 0);
        setServoPosition(servo2Pin, 180);
    }  else {
        setServoPosition(servo1Pin, 90);
        setServoPosition(servo2Pin, 90);
    }
    expression(12, 13, 11, 14, 17, 17, 17, 46, 112, 224, 192);
  } 

  delay(100);    // Pequeña pausa entre lecturas de los sensores y mediciones de distancia
}

void setServoPosition(int servoPin, int angle) {
  int pulse = map(angle, 0, 180, 102, 512);  // Mapea el ángulo a un valor de pulso entre 102 y 512
  pwm.setPWM(servoPin, 0, pulse);  // Establece el pulso correspondiente al ángulo en el canal del módulo PCA9685
}

void maxAll(int cs, int din, int clk, int reg, int col) {
  digitalWrite(cs, LOW);
  putByte(din, clk, reg);
  putByte(din, clk, col);
  digitalWrite(cs, LOW);
  digitalWrite(cs, HIGH);
}

void putByte(int din, int clk, int data) {
  byte i = 8;
  byte mask;
  while (i > 0) {
    mask = 0x01 << (i - 1);
    digitalWrite(clk, LOW);
    if (data & mask) {
      digitalWrite(din, HIGH);
    } else {
      digitalWrite(din, LOW);
    }
    digitalWrite(clk, HIGH);
    --i;
  }
}

void expression(int cs, int din, int clk, int col1, int col2, int col3, int col4, int col5, int col6, int col7, int col8) {
  writeRow(cs, din, clk, 1, col1);
  writeRow(cs, din, clk, 2, col2);
  writeRow(cs, din, clk, 3, col3);
  writeRow(cs, din, clk, 4, col4);
  writeRow(cs, din, clk, 5, col5);
  writeRow(cs, din, clk, 6, col6);
  writeRow(cs, din, clk, 7, col7);
  writeRow(cs, din, clk, 8, col8);
}

void writeRow(int cs, int din, int clk, int row, int data) {
  digitalWrite(cs, LOW);
  putByte(din, clk, row);
  putByte(din, clk, data);
  digitalWrite(cs, LOW);
  digitalWrite(cs, HIGH);
}
